#include <avr/io.h>
#include <avr/interrupt.h>  /* for sei() */
#include <util/delay.h>     /* for _delay_ms() */
#include <avr/pgmspace.h>   /* required by usbdrv.h */
#include "usbdrv.h"

#define VENDOR_RQ_GET_PEOPLE 0
#define VENDOR_RQ_GET_STAT   1
#define VENDOR_RQ_GET_VOL    2

uint8_t count_in = 0;
uint8_t count_out = 0;
uint8_t turn = 0;
uint8_t delay = 0;

/* ------------------------------------------------------------------------- */
/* ---------------------------- Utility Functions -------------------------- */
/* ------------------------------------------------------------------------- */

uint8_t in_c(uint8_t pin)
{
    // กำหนดให้ขาที่ระบุของพอร์ท C ทำหน้าที่เป็นอินพุท
    DDRC &= ~(1<<pin);
    // ดึงให้ขามีลอจิก 1 หากขาถูกปล่อยลอยไว้

    PORTC |= (1<<pin);
    // อ่านสถานะลอจิกของขา
    return ((PINC & (1<<pin))>>pin);    
} 

void out_c(uint8_t pin, uint8_t val)
{
    // กำหนดให้ขาที่ระบุของพอร์ท C ทำหน้าที่เป็นเอาท์พุท
    DDRC |= 1<<pin;
    // เซ็ตลอจิกของขาเป็น 1 ถ้า val ไม่ใช่ศูนย์ ไม่เช่นนั้นเซ็ตให้เป็น 0
    if (val)
        PORTC |= 1<<pin;
    else
        PORTC &= ~(1<<pin);
}

uint16_t read_adc(uint8_t channel)
{
    // read analog value from specified channel of port C
    ADMUX = 0x40;
    ADMUX |= channel;
    ADCSRA = 0xC6;
    while ((ADCSRA & (1<<ADSC)))
        ;
    return ADCL + ADCH*256;
}

/* ------------------------------------------------------------------------- */
/* ----------------------------- USB interface ----------------------------- */
/* ------------------------------------------------------------------------- */

usbMsgLen_t usbFunctionSetup(uchar data[8])
{
    usbRequest_t *rq = (void *)data;
    static uchar dataBuffer[3];  /* ข้อมูลนี้ต้องไม่ถูกเขียนทับหลังจาก usbFunctionSetup รีเทิร์น */

    /* ประมวลผลตามหมายเลขคำสั่งที่อยู่ใน bRequest */
    if (rq->bRequest == VENDOR_RQ_GET_STAT)
    {
        dataBuffer[0] = turn;
	usbMsgPtr = dataBuffer;
        return 1;                
    }
    else if (rq->bRequest == VENDOR_RQ_GET_VOL)
    {
	dataBuffer[0] = read_adc(0)/256;
 	dataBuffer[1] = read_adc(0)%256;
	usbMsgPtr = dataBuffer;
	return 2;
    }
    else if (rq->bRequest == VENDOR_RQ_GET_PEOPLE)
    {
	if (count_in - count_out > 0)
	{
	    dataBuffer[0] = count_in - count_out;
	} 
	else
	{
	    dataBuffer[0] = 0;
	}
	dataBuffer[1] = count_in;
	dataBuffer[2] = count_out;
	usbMsgPtr = dataBuffer;
	return 3;
    }
    return 0;   /* ไม่รู้จักคำร้องขอ ไม่ส่งข้อมูลกลับ */
}


/* ------------------------------------------------------------------------- */

int main(void)
{

    DDRC  = 0b00010000;  // PC0..PC2 = Input, PC4 = Output
    PORTC = 0b00000000;  // PC7..PC0 = 0
    PORTC |= (1 << 3);   // Enable pull-up at PC3
    DDRD  = 0b00001000;
    PORTD = 0b00000000;

    usbInit();
    usbDeviceDisconnect();  /* enforce re-enumeration, do this while interrupts are disabled! */
    _delay_ms(300);
    usbDeviceConnect();
    sei();
    for(;;){                /* main event loop */
        usbPoll();
	uint16_t light = read_adc(0);
	uint16_t led_in  = read_adc(1);
	uint16_t led_out = read_adc(2);
	if(led_in < 550)
	{
	    _delay_ms(10);
	    if(delay == 50)
	    {
		delay = 0;
	    	count_in++;
	    }
	    delay++;
	}
	if(led_out < 550)
	{
	    _delay_ms(10);
	    if(delay == 50)
	    {
		delay = 0;
	    	count_out++;
	    }
	    delay++;
	}
	if(count_in - count_out > 0)
	{
	    if(light < 600)
	    {
	    	out_c(4,1);
	    	turn = 1;
	    }
	    else
	    {
	    	out_c(4,0);
	    	turn = 0;
	    }
	}

	if(count_in - count_out == 0)
	{
	    out_c(4,0);
	    turn = 0;
	}
	if(count_out > count_in)
	{
	    count_out = count_in;
	}
    }
    return 0;
}

/* ------------------------------------------------------------------------- */
